package se.callista.microservices.core.recommendation.service;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import se.callista.microservises.core.recommendation.model.Recommendation;
import se.callista.microservises.core.recommendation.service.util.SetProcTimeBean;

import java.util.ArrayList;
import java.util.List;

/**
 * Created by magnus on 04/03/15.
 */
@RestController
public class RetweetService {

    private static final Logger LOG = LoggerFactory.getLogger(RetweetService.class);

    @Autowired
    private SetProcTimeBean setProcTimeBean;

    /*
    private int port;

    @Value("local.server.port")
    public void setPort (int port) {
        LOG.info("getReviews will be called on port: {}", port);
        this.port = port;
    }
    */

    /**
     * Sample usage: curl $HOST:$PORT/recommendation?productId=1
     *
     * @param productId
     * @return
     */
    @RequestMapping("/retweet")
    public List<Recommendation> getRetweet(
            @RequestParam(value = "tweetId",  required = true) int tweetId) {

        int pt = setProcTimeBean.calculateProcessingTime();
        LOG.info("/retweet called, processing time: {}", pt);

        sleep(pt);

        List<Retweet> list = new ArrayList<>();
        list.add(new Retweet(tweetId, 1, "Author 1", 1, "Tweet 1"));
        list.add(new Retweet(tweetId, 2, "Author 2", 2, "Tweet 2"));
        list.add(new Retweet(tweetId, 3, "Author 3", 3, "Tweet 3"));

        LOG.info("/tweet response size: {}", list.size());

        return list;
    }

    private void sleep(int pt) {
        try {
            Thread.sleep(pt);
        } catch (InterruptedException e) {
            e.printStackTrace();
        }
    }

    /**
     * Sample usage:
     *
     *  curl "http://localhost:10002/set-processing-time?minMs=1000&maxMs=2000"
     *
     * @param minMs
     * @param maxMs
     */
    @RequestMapping("/set-processing-time")
    public void setProcessingTime(
        @RequestParam(value = "minMs", required = true) int minMs,
        @RequestParam(value = "maxMs", required = true) int maxMs) {

        LOG.info("/set-processing-time called: {} - {} ms", minMs, maxMs);

        setProcTimeBean.setDefaultProcessingTime(minMs, maxMs);
    }
}
